#!/bin/csh -f

# $Id: concat_geosarl.csh,v 1.1 2013/06/19 17:46:51 trn Exp $
# Simple script that prepares GEOS d01.yyyymmdd.arl files
# See help message for details

switch ($#argv)
case 0:
	echo "$0 aborting because supplied $#argv arguments"
        echo "This script prepares GEOS d01.yyyymmdd.arl files, containing:"
        echo "  hours 06-21 of day=yyyymmdd, and hours 00-06 of the following day"
        echo "  (analogous to WRF arl files, enabling run_receptors.pl Stilt runs"
        echo "   using 'd01' as the metsource parameter)"
        echo "Run this script after running run_geos2arl.csh and preconcat_geosarl.csh"
	echo "It needs 1 or more arguments:"
	echo "1 or more - idays (yyyymmdd)"
        echo "Environment variables:"
        echo "  GRESTORED_ARL (REQ) - parent directory of arl/yyyymmdd working directories and Gmetfiles directory"
        echo "  GMETFILES (opt) - destination directory of d01.yyyymmdd.arl files, defaults to GRESTORED_ARL/Metfiles"
	exit (1)
default:
endsw

if ($?GRESTORED_ARL) then
 set arldir=${GRESTORED_ARL}/arl
else
 echo "Missing required environment variables GRESTORED_ARL"
 exit 1
endif
if ($?GMETFILES) then
 set metdir=$GMETFILES
else
 set metdir=${GRESTORED_ARL}/Metfiles
endif

echo "Running $0 with:"
echo " GRESTORED_ARL=$GRESTORED_ARL"
echo " arldir=$arldir"
echo " metdir=$metdir"

set prevdir=""
foreach iday ( $* )
  @ idom=0
  set domain="d01"
  echo ""
  echo "Processing iday=$iday, domain=$domain"
  cd $arldir/$iday
  if ("a$prevdir" == "a") then
    echo "Not linking to previous iday forecast"
  else
    set prevfile=`/bin/ls -1 ../$prevdir/DATA_????????_????.GEOS | tail -1`
    echo "ln -s $prevfile ."
    ln -s $prevfile .
  endif
  if (-e ${metdir}/${domain}.${iday}.arl) then
    echo "Not creating ${metdir}/${domain}.${iday}.arl, it already exists"
  else
    echo "Creating ${metdir}/${domain}.${iday}.arl"
    cat DATA_????????_????.GEOS > ${metdir}/${domain}.${iday}.arl
  endif
  set prevdir=$iday
end

